################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.component import adapter, getUtility, getMultiAdapter, queryMultiAdapter
from zope.app.container.interfaces import INameChooser
from plone.portlets.interfaces import IPortletAssignmentMapping, ILocalPortletAssignmentManager, IPortletManager

from plone.app.portlets.portlets import navigation

from Products.Archetypes.interfaces import IObjectInitializedEvent
from Products.CMFCore.utils import getToolByName

from ..interfaces import IInstitution, IDepartment
from ..portlets import institutionportlet
from ..portlets import departmentportlet
from ..portlets import equalinstitutionsportlet
from ..portlets import equaldepartmentsportlet

import logging
log = logging.getLogger('vs.org')

def setup_institution_portlets(obj, event):

    ptool = getToolByName(obj,'portal_url')
    c_uid = obj.UID()
    c_root =  '/' + '/'.join(ptool.getRelativeContentPath(obj))

    #left portlets
    manager = getUtility(IPortletManager, name=u'plone.leftcolumn')
    mapping = getMultiAdapter((obj, manager,), IPortletAssignmentMapping)

    has_navigation = False
    has_equal_institutions = False

    for value in mapping.values():
        if value.__name__ == 'navigation':
            has_navigation= True
        elif value.__name__ == 'label_equalinstitutionsportlet':
            has_equal_institutions = True

    if not has_navigation:
        assignment_navigation = navigation.Assignment(name=obj.Title(), root=c_root, currentFolderOnly=False, includeTop=False, topLevel=0, bottomLevel=2)

    if not has_equal_institutions:
        assignment_equalinstitutions = equalinstitutionsportlet.EqualInstitutionsPortletAssignment(institution_uid=c_uid)

    chooser = INameChooser(mapping)

    if not has_navigation:
        mapping[chooser.chooseName(None, assignment_navigation)] = assignment_navigation
    if not has_equal_institutions:
        mapping[chooser.chooseName(None, assignment_equalinstitutions)] = assignment_equalinstitutions

    # dont aquire portlets at left column
    assignable = queryMultiAdapter((obj, manager), ILocalPortletAssignmentManager)
    assignable.setBlacklistStatus('context', True)

    #right portlets 
    manager = getUtility(IPortletManager, name=u'plone.rightcolumn')
    mapping = getMultiAdapter((obj, manager,), IPortletAssignmentMapping)


    has_institution = False
    for value in mapping.values():
        if value.__name__ == 'label_institutionportlet':
            has_institution = True

    if not has_institution:
        assignment_institutionportlet = institutionportlet.InstitutionPortletAssignment(institution_uid=c_uid)
        chooser = INameChooser(mapping)
        mapping[chooser.chooseName(None, assignment_institutionportlet)] = assignment_institutionportlet

    # dont aquire portlets at right column
    assignable = queryMultiAdapter((obj, manager), ILocalPortletAssignmentManager)
    assignable.setBlacklistStatus('context', True)

#@adapter(IDepartment, IObjectInitializedEvent)
def setup_department_portlets(obj, event):

    # equaldepartments-portlet 
    manager = getUtility(IPortletManager, name=u'plone.leftcolumn')
    mapping = getMultiAdapter((obj, manager,), IPortletAssignmentMapping)

    has_equaldepartments = False
    for value in mapping.values():
        if value.__name__ == 'label_equaldepartmentsportlet':
            has_equaldepartments = True

    if not has_equaldepartments:
        assignment_equaldepartments = equaldepartmentsportlet.EqualDepartmentsPortletAssignment(department_uid=obj.UID())
        chooser = INameChooser(mapping)
        equaldepartments_name  = chooser.chooseName(None, assignment_equaldepartments)
        mapping[ equaldepartments_name] = assignment_equaldepartments

    # departmentportlet 
    manager = getUtility(IPortletManager, name=u'plone.rightcolumn')
    mapping = getMultiAdapter((obj, manager,), IPortletAssignmentMapping)

    has_department = False
    for value in mapping.values():
        if value.__name__ == 'label_departmentportlet':
            has_department = True
    if not has_department:
        assignment_department = departmentportlet.DepartmentPortletAssignment(department_uid=obj.UID())
        chooser = INameChooser(mapping)
        department_name  = chooser.chooseName(None, assignment_department)
        mapping[ department_name] = assignment_department




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class InstitutionView(BrowserView):
    """Default view, one institution """

    __call__ = ViewPageTemplateFile('institution_view.pt')




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: organisation.py 236202 2011-03-19 17:33:04Z schiele $

from Acquisition import aq_inner
from Acquisition import aq_parent
 
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.PythonScripts.standard import url_quote_plus

from plone.memoize.instance import memoize
from zope.component import getUtility
from zope.interface import implements

from ..interfaces import IKeywordExtractor
from .interfaces import IOrganisation
from .departmentoverview import DepartmentOverview
from .institutionoverview import InstitutionOverview


class Organisation(BrowserView):
    """Default view, all institutions """

    implements(IOrganisation)

    @memoize
    def getInstitutionOverview(self):
        """
        """
        i_view = InstitutionOverview(self.context.getInstitution(), self.request)
        return i_view.getData()
    
    @memoize
    def getDepartmentOverview(self):
        """
        """
        i_view = DepartmentOverview(self.context, self.request)
        return i_view.getData()

    def getLocationOverview(self):
        """
        """
        extractor = getUtility(IKeywordExtractor)
        path = '/'.join(aq_parent(aq_inner(self.context)).getPhysicalPath())
        locations = extractor.subjects(aq_parent(aq_inner(self.context)), path, query_index='country', type='Institution', sort_on='sortable_title')
       
        ptool = getToolByName(self.context, 'plone_utils')
        folder_path = aq_parent(aq_inner(self.context)).absolute_url()
        for location in locations:
            tmp = {}
            for brain in location['brains']:
                if brain['province'] not in tmp.keys():
                    normalized = ptool.normalizeString(brain['province']) 
                    tmp[brain['province']] = {'province': brain['province'],'link': "%s/%s" %(folder_path, normalized) ,'brains':[]}
                tmp[brain['province']]['brains'].append(brain)
            for key in tmp.keys():
                tmp[key]['brains'].sort(lambda x, y : cmp(x['Title'].strip().lower(),y['Title'].strip().lower()))
            
            del location['brains']        
            
            t_keys = tmp.keys()
            t_keys.sort() 
            location['provinces'] = [tmp[key] for key in t_keys]

        return locations

    def getDataByBusinessArea(self):

        catalog = getToolByName(self.context, 'portal_catalog')
        areas = catalog.uniqueValuesFor('BusinessArea')

        area2inst = dict()
        for area in areas:
            institutions = list()
            for brain in catalog(portal_type='Institution',
                                 BusinessArea=area,
                                 sort_on='sortable_title'):
                institutions.append(brain.getObject())
            area2inst[area] = institutions

        return dict(areas=areas,
                    area2inst=area2inst)

    def getBusinessAreasForCountryRegion(self):

        catalog = getToolByName(self.context, 'portal_catalog')
        areas = catalog.uniqueValuesFor('BusinessArea')
        country_region = self.request.country_region.lower()

        areas_in_region = list()
        for area in areas:
            for brain in catalog(portal_type='Institution', BusinessArea=area):
                inst = brain.getObject()
                inst_ref = ('%s-%s' % (inst.getCountry(), inst.getProvince())).lower()
                if inst_ref == country_region:
                    areas_in_region.append(dict(area=area, id=area))
                    break
          
        return areas_in_region

    def getInstitutionsInLocationForBusinessArea(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        area = self.request.area
        country_region = self.request.country_region.lower()

        result = list()
        for brain in catalog(portal_type='Institution', BusinessArea=area):
            inst = brain.getObject()
            inst_ref = ('%s-%s' % (inst.getCountry(), inst.getProvince())).lower()
            if inst_ref == country_region:
                result.append(inst)

        return result

    def newId(self):
        count = 0
        while True:
            count += 1
            yield 'node-%d' % count




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.component import getUtility
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName

def InstitutionVocabulary(context):
    """ Return all institutions as vocabulary """

    site = getUtility(ISiteRoot)
    catalog = getToolByName(site, 'portal_catalog')
    terms = list()
    for brain in catalog(portal_type='Institution', sort_on='sortable_title'):
        terms.append(SimpleTerm(value=brain.UID,
                                token=brain.UID,
                                title=brain.Title))
    return SimpleVocabulary(terms)



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: __init__.py 2981 2011-03-16 10:54:12Z carsten $

from zope.interface import Interface

# -*- extra stuff goes here -*-
from base import IBase
from employee import IEmployee
from employeefolder import IEmployeefolder
from department import IDepartment
from institution import IInstitution
from vcard import IVCardData
from addresses import IAddressProvider
from keywords import IKeywordExtractor




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.interface import implements
from ..interfaces import IAddressProvider

class InstitutionAddresses(object):
    """ 
    Adapt an institution to address data
    """

    implements(IAddressProvider)

    def __init__(self, context):
        self.context = context

    def getAddresses(self):
        """ """
        result = []
        for address in self.context.getAddresses():
            if address['street']:
                result.append( {
                    'title': "%s, %s %s" % (address['street'], address['zipCode'], address['city']), 
                    'geocode': address['geocode'] and [float(i.strip()) 
                                                       for i in address['geocode'].split(',')] or [],
                })
        return result



###############################################################
# vs.org (C) 2011, Veit Schiele
################################################################


import unittest2

from zope.component import getUtility
import zope.schema.interfaces

from base import TestBase

class InstitutionTests(TestBase):

    def _makeOne(self):
        self.portal.invokeFactory('Institution', id='zopyxgroup')
        return self.portal['zopyxgroup']

    def testCatalogSetup(self):
        catalog = self.portal.portal_catalog
        indexes = catalog.indexes()
        self.assertEqual('position' in indexes, True)
        self.assertEqual('BusinessArea' in indexes, True)
        self.assertEqual('lastname' in indexes, True)
        schema = catalog.schema()
        self.assertEqual('position' in schema, True)
        self.assertEqual('BusinessArea' in schema, True)
        self.assertEqual('lastname' in schema, True)

    def testSetupPropertysheet(self):
        self.assertEqual('vsorg_properties' in self.portal.portal_properties.objectIds(), True)
        vsorg_properties = self.portal.portal_properties.vsorg_properties.objectIds()
#        self.assertEqual('numberRegex' in vsorg_properties, True)

    def testCreateInstitution(self):
        self.login()
        inst = self._makeOne()
        self.assertEqual(inst.portal_type, 'Institution')
        self.assertEqual(len(inst.getNumbersFiltered()), 0)

    def testNumbers(self):
        self.login()
        inst = self._makeOne()
        numbers = [
                dict(number='0049-30-8185667-1'),
                dict(number='foo'),
                ]
        inst.setNumbers(numbers)

    def testGoogleMapsViews(self):
        self.login()
        inst = self._makeOne()
        inst.REQUEST['ACTUAL_URL'] = 'http://foo' # workaround

        js_view = inst.restrictedTraverse('@@maps.js', None)
        self.assertEqual(js_view is not None, True)
        html = js_view()

        view = inst.restrictedTraverse('@@map', None)
        self.assertEqual(view is not None, True)
        html = view()


class InstitutionPortletTests(TestBase):

    def _makeOne(self):
        self.portal.invokeFactory('Institution', id='zopyxgroup', title='zopyx group')
        return self.portal['zopyxgroup']

    def testVocabulary(self):
        self.login()
        # The vocabulary factory
        service = getUtility(zope.schema.interfaces.IVocabularyFactory, name='source_institutions')
        # no institutions so far
        vocabulary = service(self.portal)
        self.assertEqual(len(vocabulary), 0)
        # create an institution and then check the lenght of the vocabulary
        inst = self._makeOne()
        vocabulary = service(self.portal)
        self.assertEqual(len(vocabulary), 1)
        first_term = [x for x in vocabulary][0] # should point to ZG institution
        self.assertEqual(first_term.title, 'zopyx group')
        self.assertEqual(first_term.token, inst.UID())


def test_suite():
    from unittest2 import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(InstitutionTests))
    suite.addTest(makeSuite(InstitutionPortletTests))
    return suite



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################


import unittest2
from AccessControl.SecurityManagement import newSecurityManager
from base import TestBase

class DepartmentTests(TestBase):

    def _makeOne(self):
        self.portal.invokeFactory('Institution', id='zopyxgroup')
        inst = self.portal['zopyxgroup']
        inst.invokeFactory('Department', id='banality')
        return inst['banality']

    def testCreateDepartment(self):
        self.login()
        dept = self._makeOne()
        self.assertEqual(dept.portal_type, 'Department')

    def testCreateDepartmentOutsideInstitution(self):
        self.login()
        # raises ValueError(Disallowed portal_type ..)
        with self.assertRaises(ValueError):
            self.portal.invokeFactory('Department', id='banality')



def test_suite():
    from unittest2 import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(DepartmentTests))
    return suite



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################


import unittest2
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import quickInstallProduct
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import setRoles, login
from plone.testing import z2

from zope.configuration import xmlconfig
from AccessControl.SecurityManagement import newSecurityManager

import vs.org
import Products.ATVocabularyManager
import Products.DataGridField
import Products.MasterSelectWidget
try:
    import Products.LinguaPlone
    HAVE_LP = True
except ImportError:
    HAVE_LP = False
        
from AccessControl.SecurityManagement import newSecurityManager

class VsOrgFixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        xmlconfig.file('configure.zcml', Products.DataGridField, context=configurationContext)
        xmlconfig.file('configure.zcml', Products.MasterSelectWidget, context=configurationContext)
        xmlconfig.file('configure.zcml', Products.ATVocabularyManager, context=configurationContext)
        if HAVE_LP:
            xmlconfig.file('configure.zcml', Products.LinguaPlone, context=configurationContext)
        xmlconfig.file('configure.zcml', vs.org, context=configurationContext)

        # Install product and call its initialize() function
        z2.installProduct(app, 'Products.DataGridField')
        z2.installProduct(app, 'Products.ATVocabularyManager')
        z2.installProduct(app, 'Products.MasterSelectWidget')
        z2.installProduct(app, 'Products.DataGridField')
        if HAVE_LP:
            z2.installProduct(app, 'Products.LinguaPlone')
        z2.installProduct(app, 'vs.org')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        quickInstallProduct(portal, 'Products.DataGridField')
        quickInstallProduct(portal, 'Products.ATVocabularyManager')
        quickInstallProduct(portal, 'Products.MasterSelectWidget')
        if HAVE_LP:
            quickInstallProduct(portal, 'Products.LinguaPlone')
        applyProfile(portal, 'vs.org:default')
        portal.acl_users.userFolderAddUser('god', 'dummy', ['Manager'], []) 
        setRoles(portal, 'god', ['Manager'])
        login(portal, 'god')

    def tearDownZope(self, app):
        # Uninstall product
        z2.uninstallProduct(app, 'vs.org')


VS_ORG_FIXTURE = VsOrgFixture()
VS_ORG_INTEGRATION_TESTING = IntegrationTesting(bases=(VS_ORG_FIXTURE,), name="VsOrgFixture:Integration")

class TestBase(unittest2.TestCase):

    layer = VS_ORG_INTEGRATION_TESTING

    @property
    def portal(self):
        return self.layer['portal']

    def login(self):
        """ Login as manager """
        user = self.portal.acl_users.getUser('god')
        newSecurityManager(None, user.__of__(self.portal.acl_users))




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

import re

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import View
from compatible import atapi 

from .. import orgMessageFactory as _

class NumbersMixin(object):

    security = ClassSecurityInfo()

    security.declareProtected(View, 'getNumbersFiltered')
    def getNumbersFiltered(self):
        return [d for d in self.getNumbers() if d['number']]

    def _getFirstNumber(self, type='phone'):
        for d in self.getNumbers():
            if d['type'] == type:
                return d['number']
        return None

    security.declareProtected(View, 'getTelephone')
    def getTelephone(self):
        """ return Telephone from DGF """
        return self._getFirstNumber('phone')

    security.declareProtected(View, 'getMobile')
    def getMobile(self):
        """ return Mobile from DGF """
        return self._getFirstNumber('mobile')

    security.declareProtected(View, 'getFax')
    def getFax(self):
        """ return Mobile from DGF """
        return self._getFirstNumber('fax')

    security.declareProtected(View, 'getNumbersByType')
    def getNumbersByType(self, type='phone', do_filter=True):
        """ Return all numbers for a given type.

            If 'do_filter' is set: expose only numbers that are either
            'externally_visible' or when the member has on of the roles
            Authenticated or Member.
        """

        mt = getToolByName(self, 'portal_membership')
        member = mt.getAuthenticatedMember()
        roles = member.getRolesInContext(self)
        view_internal_numbers = False
        if 'Authenticated' in roles or 'Member' in roles:
            view_internal_numbers = True
        result = list()
        for d in self.getNumbers():
            # Check DGF flag (''==only internally visible, '1'==externally visible)
            number_is_internal = (d.get('externally_visible', '')=='')
            if d['type'] == type and d['number']:
                if do_filter:
                    if not number_is_internal or (number_is_internal and view_internal_numbers):

                        result.append(d['number'])
                else:
                    result.append(d['number'])

        return result

    security.declareProtected(View, 'getNumberTypes')
    def getNumberTypes(self):
        """ return all number types """
        result = list()
        for d in self.getNumbers():
            if not d['type'] in result and d['number']:
                result.append(d['type'])
        return result

    def validate_numbers(self, rows):
        """ Numbers validation """

        pprop = getToolByName(self, 'portal_properties')
        vsprop = getToolByName(pprop, 'vsorg_properties')
        number_regex = re.compile(vsprop.numberRegex)
        for d in rows:
            if not number_regex.match(d['number']):
                return _('Invalid number: %s' % d['number'])

    security.declarePublic('getNumbersVocabulary')
    def getNumbersVocabulary(self):
        """return number vocabulary for DGF (managed through ATVocabularyManager)"""
        vocab_tool = getToolByName(self, 'portal_vocabularies')
        vocab = vocab_tool[self.numbers_vocabulary].getVocabularyDict()
        lst = list()
        for k,v in vocab.items(): 
            lst.append((k, v))
        return atapi.DisplayList(lst)



# -*- coding: utf-8 -*-

################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.interface import Interface

class IStaffSortView(Interface):
    """ StaffSortView
    """
    def getSortedStaff(self):
        """
        """
class IImageView(Interface):
    """ Image - View
    """
    def tag(self,fieldid=None,scale=None, height=None, width=None, alt=None,css_class=None, title=None, **kwargs):
        """
        """
    def getImageSize(self, fieldid=None, scale=None):
        """
        """

class IOrganisation(Interface):
    """
    """
    def getInstitutionOverview(self):
        """
        """
    def getDepartmentOverview(self):
        """
        """

class IVCardView(Interface):

    def vcard(self, **kwargs):
        pass

class IMapView(Interface):
    """ Google Maps view  """

    def getAddresses(self):
        """ Addresses to be displayed  """

class IVBBView(Interface):
    """ VBB - Link
    """
    def getVBBUrl(self):
        """
        """




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: institution.py 2987 2011-03-16 14:55:06Z carsten $

"""
Definition of the Institution content type
"""

from AccessControl import ClassSecurityInfo
from zope.interface import implements

from archetypes.referencebrowserwidget.widget import ReferenceBrowserWidget
from pycountry import pycountry as pycountry
from compatible import atapi
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from Products.MasterSelectWidget.MasterSelectWidget import MasterSelectWidget

from .. import orgMessageFactory as _
from ..interfaces import IBase
from ..interfaces import IInstitution
from ..config import PROJECTNAME
from base import BaseSchema
from numbers import NumbersMixin

from Products.ATVocabularyManager.namedvocabulary import NamedVocabulary
from Products.DataGridField import DataGridField, DataGridWidget, FixedRow
from Products.DataGridField.Column import Column

# define slave parameters for master
slave_fields = ({'name': 'province',
                 'action': 'vocabulary',
                 'vocab_method': 'getProvinceVocabulary',
                 'control_param': 'master',
                 },)

InstitutionSchema = folder.ATFolderSchema.copy() + ConstrainTypesMixinSchema.copy() + BaseSchema.copy() + atapi.Schema((

    atapi.TextField('text',
        required=False,
        searchable=True,
        primary=True,
        storage = atapi.AnnotationStorage(),
        validators = ('isTidyHtmlWithCleanup',),
        default_output_type='text/x-html-safe',
        widget = atapi.RichWidget(
            description='',
            label=_(u'label_body_text', default=u'Body Text'),
            rows=25,
            allow_file_upload=zconf.ATDocument.allow_document_upload),
    ),

    atapi.ReferenceField(
        name='employees',
        multiValued=1,
        allowed_types=('Employee',),
        relationship='isEmployee',
        widget=ReferenceBrowserWidget(
            show_review_state=1,
            allow_sorting=1,
            label=_(u'label_employees',),
        ),
    ),

    atapi.LinesField(
        'businessArea',
        accessor='BusinessArea',
        multiValued=1,
        searchable=True,
        storage=atapi.AnnotationStorage(),
        widget=atapi.KeywordWidget(
            label=_(u"label_businessArea"),
            description=_(u"help_businessArea"),
        ),
    ),


    DataGridField(
        'addresses',
        searchable=True,
        allow_empty_rows=False,
        allow_oddeven=True,
        storage=atapi.AnnotationStorage(),
        columns=('street','poBox','zipCode','city', 'geocode'),
        widget=DataGridWidget(
            label=_(u'label_addresses',),
            description=_(u'help_addresses',),
            columns = {
                    'street':   Column(_(u'label_street')),
                    'poBox':    Column(_(u'label_poBox')),
                    'zipCode':  Column(_(u'label_zipCode')),
                    'city':     Column(_(u'label_city')),
                    'geocode':  Column(_(u'label_geocode')),
            },
        ),
        validators=('isVSAddress',),
    ),


    atapi.StringField(
        'country',
        required=True,
        default='Select country...',
        vocabulary='getCountryVocabulary',
        storage=atapi.AnnotationStorage(),
        # could avoid importing, alas does not work
        # i18n_domain='plone',
        widget=MasterSelectWidget(
            slave_fields=slave_fields,
            label=_(u"label_country"),
            description=_(u"help_country"),
       ),
    ),


    atapi.StringField(
        'province',
        required=True,
        default='',
        vocabulary=[],
        storage=atapi.AnnotationStorage(),
        widget=atapi.SelectionWidget(
            label=_(u"label_province"),
            description=_(u"help_province"),
            format='select',
        ),
    ),


    atapi.TextField(
        'directions',
        required=False,
        searchable=True,
        storage=atapi.AnnotationStorage(),
        validators=('isTidyHtmlWithCleanup',),
        default_output_type='text/x-html-safe',
        widget=atapi.RichWidget(
            label=_(u"label_directions"),
            description=_(u"help_directions"),
        ),
    ),

))

InstitutionSchema['title'].storage = atapi.AnnotationStorage()
InstitutionSchema['description'].storage = atapi.AnnotationStorage()
InstitutionSchema['text'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    InstitutionSchema,
    folderish=True,
    moveDiscussion=False
)

class Institution(folder.ATFolder, NumbersMixin):
    """Institution with addresses, phone numbers etc."""
    security = ClassSecurityInfo()
    implements(IInstitution, IBase)
    meta_type = "Institution"
    numbers_vocabulary = 'number_types_institution'
    schema = InstitutionSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    directions = atapi.ATFieldProperty('directions')
    addresses = atapi.ATFieldProperty('addresses')
    businessArea = atapi.ATFieldProperty('businessArea')
    country = atapi.ATFieldProperty('country')
    province = atapi.ATFieldProperty('province')

    security.declareProtected(View, 'getInstitution')
    def getInstitution(self):
        """ return Institution """
        return self

    def at_post_edit_script(self, *args, **kw):
        """ Filter out empty number fields """
        super(Institution, self).at_post_edit_script(*args, **kw)
        numbers = self.getNumbers()
        numbers = [d for d in numbers if d['number']]
        self.setNumbers(numbers)

    def getCountryVocabulary(self):
        """ Return countries from pycountry. The country list
            can be filtered by adding the related country codes to
            portal_properties.vsorg_properties.allowedCountryCodes.
        """

        pprop = getToolByName(self, 'portal_properties')
        vsprop = getToolByName(pprop, 'vsorg_properties')
        allowed_codes = vsprop.allowedCountryCodes
        allowed_codes = [code.lower().strip() for code in allowed_codes]
        result = atapi.DisplayList()
        result.add(u'', _(u'please_select_a_country'))
        for country in pycountry.countries:
            if allowed_codes and not country.alpha2.lower() in allowed_codes:
                continue
            result.add(country.alpha2, country.name)
        return result

    def getProvinceVocabulary(self, master):
        """ Return corresponding provinces """ 
        try: 
            subdivisions = list(pycountry.subdivisions.get(country_code=master))
        except KeyError:
            subdivisions = []
        result = list()
        for subdiv in subdivisions:
            result.append((subdiv.code, subdiv.name))
        result.sort(lambda x,y: cmp(x[1], y[1]))
        result.insert(0, (u'', _(u'please_select_a_province')))
        return atapi.DisplayList(result)

atapi.registerType(Institution, PROJECTNAME)



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

import urllib
from cgi import escape

from interfaces import IVBBView
from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from plone.memoize.instance import memoize

class ImageView(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    # TODO: eventuell nicht default image sonder im schema nach dem ersten image suchen ?
    def tag(self,fieldid=None,scale=None, height=None, width=None, alt=None,css_class=None, title=None, **kwargs):
        """ tag
        """
        if fieldid is None:
            fieldid = 'image'
        field = self.context.getField(fieldid)
        image = field.getScale(self.context, scale=scale)
        if image:
            img_width, img_height = field.getSize(self.context, scale=scale)
        else:
            img_height=0
            img_width=0

        if height is None:
            height=img_height
        if width is None:
            width=img_width

        url = self.context.absolute_url()
        url+= '/image_' + fieldid
        if scale:
            url+= '_' + scale

        values = {
            'src' : url,
            'alt' : escape(alt and alt or self.context.Title(), 1),
            'title' : escape(title and title or self.context.Title(), 1),
            'height' : height,
            'width' : width,
        }

        result = '<img src="%(src)s" alt="%(alt)s" title="%(title)s" '\
            'height="%(height)s" width="%(width)s"' % values

        if css_class is not None:
            result = '%s class="%s"' % (result, css_class)

        for key, value in kwargs.items():
            if value:
                result = '%s %s="%s"' % (result, key, value)

        return '%s />' % result
        # return self.context.getField('image').tag(self.context , **kwargs)

    def getImageSize(self, fieldid=None, scale=None):
        """ image size """
        if fieldid is None:
            fieldid = 'image'
        field = self.context.getField(fieldid)
        return field.getSize(self.context,scale=scale)

    def getReferenceImage(self, fieldid=None):
        """ return referenced Image """
        return self.context.getImageReference()




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from plone.memoize.instance import memoize

from zope.interface import implements
from AccessControl import getSecurityManager
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import View
from Products.Five.browser import BrowserView

from .interfaces import IStaffSortView
from ..content import employeefolder

class StaffSortView(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getSortedStaff(self):
        """ Return sorted Userlist """

        result = []
        users = list(self.context.getEmployees())
        for user in users:
            c_dic = {
                'degree':       user.getDegree(),
                'degree_after':       user.getDegreeAfter(),
                'firstname':    user.getFirstname(),
                'lastname':     user.getLastname(),
                'position':     user.getPosition(),
                'url':         user.absolute_url(),
                'DisplayNamePosition':  user.DisplayNamePosition(),
            }
            result.append(c_dic)
        return result

    # The 'available' property is used to determine if the portlet should
    # be shown.

    @property
    def available(self):
        return len(self._data()) > 0

    # To make the view template as simple as possible, we return dicts with
    # only the necessary information.

    def employeefolder(self):
        for brain in self._data():
            employee = brain.getObject()
            yield dict(title=registrant.name,
                       url=brain.getURL())

    # By using the @memoize decorator, the return value of the function will
    # be cached. Thus, calling it again does not result in another query.
    # See the plone.memoize package for more.

    @memoize
    def _data(self):
        context = aq_inner(self.context)
        limit = self.data.count

        query = dict(object_provides = IEmployee.__identifier__)

        query['sort_on'] = 'modified'
        query['sort_order'] = 'reverse'
        query['sort_limit'] = limit
        if not self.data.sitewide:
            query['path'] = '/'.join(context.getPhysicalPath())

        # Ensure that we only get active objects, even if the user would
        # normally have the rights to view inactive objects (as an
        # administrator would)
        query['effectiveRange'] = DateTime()

        catalog = getToolByName(context, 'portal_catalog')
        results = catalog(query)
        employeefolder = results[:limit]

        return employeefolder



from zope import schema
from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.memoize import ram
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.cache import render_cachekey

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from .. import orgMessageFactory as _

class IEqualDepartmentsPortlet(IPortletDataProvider):
    """
    """
    department_uid = schema.TextLine(
        title    = _(u'label_uid'),
        required = True,
    ) 
    
class EqualDepartmentsPortletAssignment(base.Assignment):
    implements(IEqualDepartmentsPortlet)
    department_uid = '' 

    def __init__(self, department_uid=''):
        self.department_uid = department_uid
    
    @property
    def title(self):
        return _(u'label_equaldepartmentsportlet')

class Renderer(base.Renderer):

    _template = ViewPageTemplateFile('equaldepartmentsportlet.pt')

    # @ram.cache(render_cachekey)
    def render(self):
        return xhtml_compress(self._template())

    @property
    def available(self):
        data = self.getOtherDepartments()
        for item in data:
            if item['departments'] :    
                return True
        return False
    
    
    def getOtherDepartments(self):
        dep_data = self.getDepartmentData()
        context = aq_inner(self.context)
        catalog = getToolByName(context, 'portal_catalog')
        items = []
        for spec in dep_data['specialisations']:
            items.append({'spec': spec, 'departments': []})
            deps = catalog(Specialisation=spec, Type='Department')
            for dep in deps:
                if dep.UID != self.data.department_uid: # ignore ourself
                    institution = dep.getObject().getInstitution()
                    dep_dic = {
                        'institutionTitle': institution.Title(),    
                        'departmentLink': dep.getURL(), 
                        'departmentTitle': dep.Title ,  
                        'departmentDescription': dep.Description   
                    }
                    items[-1]['departments'].append(dep_dic)
        return items
            

    def getDepartmentData(self):
        dep_data = {'specialisations': [], 'path': ''}
        context = aq_inner(self.context)
        catalog = getToolByName(context, 'portal_catalog')
        uid = self.data.department_uid
        if uid:
             result = list(catalog(UID=uid,))
             if len(result):
                dep_data['specialisations'] = list(result[0].Specialisation)
                dep_data['path'] = result[0].getPath()
                
        return dep_data 
 
class AddForm(base.NullAddForm):
    form_fields = form.Fields(IEqualDepartmentsPortlet)

    def create(self):
        return EqualDepartmentsPortletAssignment(department_uid=self.context.UID())





################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: views.py 2981 2011-03-16 10:54:12Z carsten $

from AccessControl import getSecurityManager
from BeautifulSoup import BeautifulSoup, Tag
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from .. import orgMessageFactory as _

class EmployeeView(BrowserView):
    """
    """

    def _cleanupHtml(self, html):
        """ Get hold of the main content of the HTML documents and strip
            of unwanted information.
        """

        soup = BeautifulSoup(html)
        div = soup.find('div', {'class' : 'documentContent'})
        for item in ( ('div', {'class' : 'documentByLine'}), 
                      ('div', {'class' : 'documentActions'}),
                      ('div', {'class' : 'reviewHistory'}),
                      ('h1', )):
            for e in div.findAll(*item):
                e.extract()
        return div.renderContents()

    def _canView(self, obj):
        user = getSecurityManager().getUser()
        return user.has_permission('View', obj)




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################


import unittest2
from base import TestBase

NUMBERS = [
    dict(type='phone', number='+49 123 45678-00', externally_visible='1'),
    dict(type='fax', number='+49 123 45678-01', externally_visible='1'),
    dict(type='phone', number='+49 123 45678-98', externally_visible=''),
    dict(type='fax', number='+49 123 45678-99', externally_visible=''),
]
class NumberMixinTests(TestBase):

    def _makeOne(self, with_numbers=True):
        self.login()
        self.portal.invokeFactory('Institution', id='zopyxgroup')
        if with_numbers:
            self.portal['zopyxgroup'].setNumbers(NUMBERS)
        return self.portal['zopyxgroup']

    def testNoNumbers(self):
        institution = self._makeOne(with_numbers=False)
        self.assertEqual(len(institution.getNumbersFiltered()), 0)

    def testWithNumbers(self):
        institution = self._makeOne()
        self.assertEqual(len(institution.getNumbersFiltered()), 4)

    def testGetPhoneNumber(self):
        institution = self._makeOne()
        phone = institution.getTelephone()
        self.assertEqual(phone, '+49 123 45678-00')

    def testGetNotExistingMobile(self):
        institution = self._makeOne()
        mobile = institution.getMobile()
        self.assertEqual(mobile, None)

    def testGetNumberTypes(self):
        institution = self._makeOne()
        types = institution.getNumberTypes()
        self.assertEqual(len(types), 2)
        self.assertEqual('phone' in types, True)
        self.assertEqual('fax' in types, True)

    def getNumbersByTypeUnfiltered(self):
        institution = self._makeOne()
        numbers = institution.getNumbersByType('phone', do_filter=False)
        self.assertEqual(len(numbers), 2)

    def getNumbersByTypeUnfiltered(self):
        institution = self._makeOne()
        numbers = institution.getNumbersByType('phone', do_filter=True)
        self.assertEqual(len(numbers), 1)

def test_suite():
    from unittest2 import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(NumberMixinTests))
    return suite



# -*- coding: iso-8859-15-*-

################################################################
# vs.org (C) 2011, Veit Schiele
################################################################


import unittest2
from base import TestBase

class EmployeeTests(TestBase):

    def _makeOne(self):
        self.portal.invokeFactory('Institution', id='veitschielecommunications')
        inst = self.portal['veitschielecommunications']
        inst.invokeFactory('Department', id='veitschiele')
        dept = inst['veitschiele']
        dept.invokeFactory('Employeefolder', id='carstenraddatz')
        employees = dept['carstenraddatz']
        employees.invokeFactory('Employee', id='craddatz', title=u'Carsten Raddatz')
        return employees['craddatz']

    def testCreateEmployee(self):
        self.login()
        employee = self._makeOne()
        self.assertEqual(employee.portal_type, 'Employee')

    def testEmployeeVCardView(self):
        self.login()
        employee = self._makeOne()
        vcard_view = employee.restrictedTraverse('@@vcard_view')
        vcard = vcard_view()
        self.assertEqual(vcard.startswith('BEGIN:VCARD'), True)
        self.assertEqual(vcard.endswith('END:VCARD\r\n'), True)

    def testEmployeFolderView(self):
        self.login()
        employee = self._makeOne()
        employee.REQUEST['ACTUAL_URL'] = 'http://foo' # workaround
        employeefolder = employee.aq_parent
        self.assertEqual(employeefolder.portal_type, 'Employeefolder')
        view = employeefolder.restrictedTraverse('@@employeefolder_view')
        result = view()


def test_suite():
    from unittest2 import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(EmployeeTests))
    return suite



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.interface import implements
from ..interfaces import IVCardData
import pycountry

class EmployeeVCard(object):
    """ 
    Adapt an employee to vcard data 
    """

    implements(IVCardData)

    def __init__(self, context):
        self.context = context

    def getVCardData(self):

        emp = self.context

        result = dict() 
        result['id'] = emp.getId()
        result['source'] = emp.absolute_url()
#        result['degree'] = emp.getDegree()
#        result['degree_after'] = emp.getDegreeAfter()
        result['degree'] = ''
        result['firstname'] = emp.getFirstname()
        result['lastname'] = emp.getLastname()
        result['position'] = emp.getPosition()
        result['institution'] = emp.getInstitution().Title()
        result['email'] = emp.getEmail()
        result['url'] = emp.getUrl()
        result['notes'] = ''

        for num_type in ('phone', 'fax', 'mobile'):
            numbers = emp.getNumbersByType(num_type)
            if numbers:
                result[num_type] = numbers[0]
            else:
                result[num_type] = None

        # full vcardname
        lst = [emp.getSalutation(),
#               emp.getDegree(),
               emp.getFirstname(),
               emp.getLastname(),
               emp.getPosition(),
               emp.getDegreeAfter(),
               ]
        lst = [item.strip() for item in lst if item.strip()]
        result['vcardname'] = ' '.join(lst)

        # image
        thumb_data = None
        image_field = emp.getField('image')
        if image_field:
            try:
                thumb_data = str(image_field.getScale(emp, 'thumb').data)
            except AttributeError:
                thumb_data = None
        result['image'] = thumb_data

        # addresses
        addresses = list()
        for ref in emp.getBRefs(relationship='isEmployee'):

            # iterate over Department and Institution addresses
            # (department addresses are more specific than institution addresses)
            orga_hierarchy  = [ref.getInstitution()]
            if ref.portal_type == 'Department':
                orga_hierarchy.insert(0, ref.getDepartment())

            for o in orga_hierarchy:
                for address in o.getAddresses():

                    # expand country and province codes first
                    country_a2 = o.getCountry()
                    try:
                        country_name = pycountry.countries.get(alpha2=country_a2).name
                    except KeyError:
                        country_name = ''

                    province_code = o.getProvince()
                    try:
                        province_name = pycountry.subdivisions.get(code=province_code).name
                    except KeyError:
                        province_name = ''

                    addresses.append(dict(zipcode=address['zipCode'],
                                          city=address['city'],
                                          street=address['street'],
                                          pobox=address['poBox'],
                                          country=country_name,
                                          province=province_name,
                                        ))
        result['addresses'] = addresses

        return result



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: employee.py 2981 2011-03-16 10:54:12Z carsten $

"""
Definition of the Employee content type
"""

from AccessControl import ClassSecurityInfo
from zope.interface import implements

from compatible import atapi
from Products.ATContentTypes.content import base
from Products.ATContentTypes.content import schemata
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn
from Products.DataGridField import DataGridField, DataGridWidget, FixedRow

from .. import orgMessageFactory as _
from ..interfaces import IBase
from ..interfaces import IEmployee
from ..config import PROJECTNAME
from base import BaseSchema
from numbers import NumbersMixin

EmployeeSchema = schemata.ATContentTypeSchema.copy() + BaseSchema.copy() + atapi.Schema((

    atapi.StringField(
        'salutation',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_salutation"),
            description=_(u"help_salutation"),
        ),
    ),

    atapi.StringField(
        'degree',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_degree"),
            description=_(u"help_degree"),
        ),
    ),

    atapi.StringField(
        'degreeAfter',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_degree_after"),
            description=_(u"help_degree_after"),
        ),
    ),


    atapi.StringField(
        'firstname',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_firstname"),
            description=_(u"help_firstname"),
        ),
    ),

    atapi.StringField(
        'middlename',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_middlename"),
            description=_(u"help_middlename"),
        ),
    ),

    atapi.StringField(
        'lastname',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_lastname"),
            description=_(u"help_lastname"),
        ),
    ),

    atapi.StringField(
        'position',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_position"),
            description=_(u"help_position"),
        ),
    ),

    atapi.ComputedField(
        'title',
        storage=atapi.AnnotationStorage(),
        widget=atapi.ComputedWidget(
            label=_(u"label_title"),
            description=_(u"help_title"),
        ),
        expression='context.DisplayName()',
        mode='r',
        accessor="Title"
    ),

    atapi.LinesField(
        'notes',
        storage=atapi.AnnotationStorage(),
        widget=atapi.LinesWidget(
            label=_(u"label_notes"),
            description=_(u"help_notes"),
        ),
    ),

))

EmployeeSchema['title'].storage = atapi.AnnotationStorage()
EmployeeSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(EmployeeSchema, moveDiscussion=False)


class Employee(base.ATCTContent, NumbersMixin):
    """Employee of an institution and/or departments"""
    security = ClassSecurityInfo()
    implements(IEmployee, IBase)
    meta_type = 'Employee'
    numbers_vocabulary = 'number_types_employee'
    schema = EmployeeSchema


    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    
    salutation = atapi.ATFieldProperty('salutation')
    degree = atapi.ATFieldProperty('degree')
    firstname = atapi.ATFieldProperty('firstname')
    middlename = atapi.ATFieldProperty('middlename')
    lastname = atapi.ATFieldProperty('lastname')
    position = atapi.ATFieldProperty('position')
    title = atapi.ATFieldProperty('title')
    notes = atapi.ATFieldProperty('notes')

    def at_post_edit_script(self, *args, **kw):
        """ Filter out empty number fields """
        super(Employee, self).at_post_edit_script(*args, **kw)
        numbers = self.getNumbers()
        numbers = [d for d in numbers if d['number']]
        self.setNumbers(numbers)

    security.declarePublic('DisplayName')
    def DisplayName(self):
        """Returns the complete Name"""
        lst = [self.getSalutation(), 
               self.getDegree(), 
               self.getFirstname(), 
               self.getMiddlename(), 
               self.getLastname()]
        displayname = ' '.join([(x or '').strip() for x in lst])
        position = self.getPosition()
        degreeAfter = self.getDegreeAfter()
        if degreeAfter:
            displayname += ", " + degreeAfter
        if position:
            displayname += ", " + position
        return displayname

    security.declarePublic('DisplayNamePosition')
    def DisplayNamePosition(self):
        """Returns the complete Name (without salutation) and position"""
        lst = [self.getDegree(),
               self.getFirstname(),
               self.getMiddlename(), 
               self.getLastname()]
        displayname = ' '.join([(x or '').strip() for x in lst])
        position = self.getPosition()
        if position:
            displayname += ", " + position
        return displayname

atapi.registerType(Employee, PROJECTNAME)



from zope import schema
from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.memoize import ram
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.cache import render_cachekey

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from .. import orgMessageFactory as _

class IEqualInstitutionsPortlet(IPortletDataProvider):
    """
    """
    institution_uid = schema.TextLine(
        title    = _(u'label_uid'),
        required = True,
    ) 
    
class EqualInstitutionsPortletAssignment(base.Assignment):
    implements(IEqualInstitutionsPortlet)
    institution_uid = '' 

    def __init__(self, institution_uid=''):
        self.institution_uid = institution_uid
    
    @property
    def title(self):
        return _(u'label_equalinstitutionsportlet')

class Renderer(base.Renderer):

    _template = ViewPageTemplateFile('equalinstitutionsportlet.pt')

    # @ram.cache(render_cachekey)
    def render(self):
        return xhtml_compress(self._template())

    @property
    def available(self):
        data = self.getOtherInstitutions()
        for item in data:
            if item['institutions'] :    
                return True
        return False
    
    def getOtherInstitutions(self):
        inst_data = self.getInstitutionData()
        context = aq_inner(self.context)
        catalog = getToolByName(context, 'portal_catalog')
        items = []
        for area in inst_data['areas']:
            items.append({'area': area, 'institutions': []})
            institutions = catalog(BusinessArea=area, Type='Institution')
            for inst in institutions:
                if inst.UID != self.data.institution_uid:
                    inst_dic = {
                        'title':        inst.Title,    
                        'url':          inst.getURL(), 
                        'description':  inst.Description 
                    }
                    items[-1]['institutions'].append(inst_dic)
        return items
            

    def getInstitutionData(self):
        inst_data = {'areas': [], 'path': ''}
        context = aq_inner(self.context)
        catalog = getToolByName(context, 'portal_catalog')
        uid = self.data.institution_uid
        if uid:
             result = list(catalog(UID=uid,))
             if len(result):
                inst_data['areas'] = list(result[0].BusinessArea)
                inst_data['path'] = result[0].getPath()
        return inst_data 
 
class AddForm(base.NullAddForm):
    form_fields = form.Fields(IEqualInstitutionsPortlet)

    def create(self):
        return EqualInstitutionsPortletAssignment(institution_uid=self.context.UID())





################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: institutionoverview.py 236195 2011-03-19 15:20:40Z schiele $

from zope.component import getUtility
from Products.Five.browser import BrowserView
from plone.memoize.view import memoize
from Products.PythonScripts.standard import url_quote_plus

from ..interfaces import IKeywordExtractor

class InstitutionOverview(BrowserView):

    def getCurrentPath(self):
        return '/'.join(self.context.getPhysicalPath())

    @memoize
    def getData(self):
        """
        Return sorted KeywordList for businessArea
        """
        extractor = getUtility(IKeywordExtractor)
        path = self.getCurrentPath()
        areas = extractor.subjects(self.context, path, query_index='BusinessArea', type='Institution', sort_on='sortable_title')
        for item in areas:
            item['link'] = url_quote_plus(item['BusinessArea'])
            current = []
            for brain in item['brains']:
               current.append(brain)
            if not current:
                del areas[areas.index(item)]
            else:
                current.sort(lambda x, y : cmp(x['Title'].strip().lower(),y['Title'].strip().lower()))
                item['brains'] = current
        return areas




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

"""
Define a browser view for the Employee content type. In the FTI
configured in profiles/default/types/Employee.xml, this is being set as the default
view of that content type.
"""

from Acquisition import aq_inner

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.memoize.instance import memoize

class EmployeeView(BrowserView):
    """Default view of an employee """

    __call__ = ViewPageTemplateFile('employee_view.pt')



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: departmentoverview.py 236234 2011-03-21 07:42:05Z ajung $

from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.memoize.view import memoize
from zope.component import getUtility

from ..interfaces import IKeywordExtractor

class DepartmentOverview(BrowserView):
    """ Default view, all departments """

    def getCurrentPath(self):
        return '/'.join(self.context.getPhysicalPath())

    @memoize
    def getData(self):
        """ return sorted KeywordList for specialisation """

        catalog = getToolByName(self.context, 'portal_catalog')
        wf_tool = getToolByName(self.context, 'portal_workflow')
        extractor = getUtility(IKeywordExtractor)
        path = self.getCurrentPath()
        specs = extractor.subjects(self.context, path, 
                                   query_index='Specialisation', 
                                   type='Department', 
                                   sort_on='sortable_title')

        result = []
        for spec in specs:
            current = {'Specialisation': spec['Specialisation'], 'deps':[]}
            
            for brain in spec['brains']:
                obj = brain.getObject()
                institution = obj.getInstitution()
                # include department only if parent institution is published
                if wf_tool.getInfoFor(institution, 'review_state') in ('published',):
                    current['deps'].append({'department': brain , 'institution': institution})

            if current['deps']:
                current['deps'].sort(lambda x, y : cmp(x['institution']['Title'].strip().lower(),y['institution']['Title'].strip().lower()))
                result.append(current)

        return result




# -*- coding: utf-8 -*-

################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: config.py 2981 2011-03-16 10:54:12Z carsten $

"""
Common configuration constants
"""

PROJECTNAME = 'vs.org'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'Employee': 'vs.org: Add Employee',
    'Employeefolder': 'vs.org: Add Employeefolder',
    'Department': 'vs.org: Add Department',
    'Institution': 'vs.org: Add Institution',
}

DEPARTMENTS_PORTLET_COLUMN = u"plone.rightcolumn"




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: setuphandlers.py 2981 2011-03-16 10:54:12Z carsten $

import os
from sets import Set
from logging import getLogger
from cStringIO import StringIO
from config import PROJECTNAME

from Products.CMFCore.utils import getToolByName
from Products.Archetypes import atapi


class Generator(object):
    def setupVocabularies(self, p, out):
        def _setup(id, filename, title_vocab):
            portal_vocab = getToolByName(p, 'portal_vocabularies')
            if id in portal_vocab.objectIds():
                portal_vocab.manage_delObjects(id)
            portal_vocab.invokeFactory('SimpleVocabulary', id=id)
            vocab = portal_vocab[id]
            vocab.setTitle(title_vocab)
            vocab.setDescription(title_vocab)
            vocab.setLanguage('')
            vocab.reindexObject()

            filename = os.path.join(os.path.dirname(__file__), 'data', filename)
            for line in file(filename):
                if not '=' in line:
                    continue
                key, value = line.split('=')
                key = key.strip()
                value = value.strip()
                vocab.invokeFactory('SimpleVocabularyTerm', id=key)
                term = vocab[key]
                term.setTitle(value)
                term.setLanguage('')
                term.reindexObject()

        _setup('number_types_employee', 'numbers_employee.txt', 'Employee')
        _setup('number_types_institution', 'numbers_institution.txt', 'Institution')
        _setup('number_types_department', 'numbers_department.txt', 'Department')
        print >> out, " Vocabularies created%s \n" 


def setupVarious(context):

    # Ordinarily, GenericSetup handlers check for the existence of XML files.
    # Here, we are not parsing an XML file, but we use this text file as a
    # flag to check that we actually meant for this import step to be run.
    # The file is found in profiles/default.

    if context.readDataFile('vs.org_various.txt') is None:
        return
    # Add additional setup code here
    out = StringIO()
    site = context.getSite()
    gen = Generator()
    gen.setupVocabularies(site, out)
    logger = context.getLogger(PROJECTNAME)
    logger.info(out.getvalue())




# -*- coding: utf-8 -*-

################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_inner

from ..interfaces import IKeywordExtractor


class KeywordExtractor(object):

    implements(IKeywordExtractor)

    def subjects(self, context, root_path, query_index='Subject', type=None, sort_on='sortable_title',depth=None):
        context = aq_inner(context)
        catalog = getToolByName(context, 'portal_catalog')
        keywords = catalog.uniqueValuesFor(query_index)
        result = []
        if not root_path:
            root_path='/'
        for keyword in keywords:
            if not type:
                if depth:
                    all = catalog({'path':{'query':root_path, 'depth': depth}, query_index: keyword , 'sort_on':sort_on})
                else:
                    all = catalog({'path':root_path, query_index: keyword ,'sort_on':sort_on})
            else:
                if depth:
                    all = catalog({'path':{'query': root_path, 'depth':depth}, query_index: keyword ,'Type':type, 'sort_on': sort_on})
                else: 
                    all = catalog({'path':root_path, query_index: keyword ,'Type':type, 'sort_on':sort_on})
            if all:
                size = len(list(all))
                current = {query_index: keyword, 'cnt': size, 'brains': all}
                result.append(current)
        return result



# -*- coding: utf-8 -*-

################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

import urllib

from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.interface import implements

from .interfaces import IVBBView

class VBBView(BrowserView):
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
                                
    @memoize
    def getVBBUrl(self):
        """ VBB Link """
        ret=u''
        country = self.context.getCountry()
#        if country not in ['de']:
#            return u''

        vbb = 'http://www.vbb-fahrinfo.de/fahrinfo/bin/query.exe/dn'
        data={}
        data['L'] ='impuls'
        to=u''

        addresses = self.context.getAddresses()
        address = None
        if len(addresses):
            address = addresses[0]

        if not address or address.get('poBox') != '':
             return

        zip = address.get('zipCode')
        city = address.get('city').decode('utf-8')
        street = address.get('street').decode('utf-8')

        if zip:
            to += """%s """ %(zip)
        if city:
            to += """%s """ %(city)
        if street:
            street=street.replace(u'Straße','Str.')
            street=street.replace(u'Strasse','Str.')
            to += street

        data['to'] = to.encode('ISO-8859-1')
        data['toType'] ='ADDRESS'
        data['REQ0HafasSearchForw'] = '0'
        data['REQ0JourneyStopsS0B'] = '5'
        data['REQ0JourneyStopsZ0B'] = '5'
        data['iER']='yes'
        if vbb:
            ret = vbb + u'?' + urllib.urlencode(data)
        return ret
    
    



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

import base64
import logging
from cStringIO import StringIO

from zope.i18n import translate
from zope.interface import implements
from zope.component import getMultiAdapter

from DateTime import DateTime
from plone.memoize.instance import memoize
from Products.Five.browser import BrowserView
from Products.ATContentTypes.lib.calendarsupport import foldLine, n2rn, vformat, rfc2445dt

from .. import config
from ..interfaces import IVCardData
from interfaces import IVCardView 

log = logging.getLogger('vs.org')

VC_HEADER = """\
BEGIN:VCARD
VERSION:3.0
PROFILE:VCARD
"""

VC_FOOTER = """\
PRODID:-// VS ORG EMPLOYEE // EMPLOYEE //DE
END:VCARD
"""

from_encoding = 'utf-8'
to_encoding = 'windows-1252'

class VCardView(BrowserView):
    
    implements(IVCardView)
    
    def _u(self, val):
        return unicode(str(val),from_encoding).encode(to_encoding) 

    def _vf(self, val):
        return vformat(self._u(val))

    def getVCard(self):
        """ vCard """

        VF = self._vf # shortcut
        out = StringIO()           

        # extract the vcard relevant data through an IVCardData adapter
        adapter = IVCardData(self.context)
        data = adapter.getVCardData()

        # auf foldLine mit absicht verzichtet ... buggy is das adressbuch von
        # MacOS -> kann keine karte einlesen, in der foldline benutzt wird :(

        # NAME:Hans Mustermann Visitenkarte
        out.write('NAME:%s Vcard\n' % vformat(data['id']))
        # SOURCE
        out.write('SOURCE:%s\n' % data['source'])
        # FN
        out.write('FN;LANGUAGE=de;CHARSET=%s:%s\n' % 
                  (to_encoding, VF(data['vcardname'])))
        # N:nachname;vorname;2.vorname;titel
        content = '%s;%s;;%s' % (VF(data['lastname']), 
                                   VF(data['firstname']), 
                                   VF(data['degree']))
        out.write('N;LANGUAGE=de;CHARSET=%s:%s\n' % (to_encoding, content))
        # SORT-STRING

        if data['lastname']:
            out.write('SORT-STRING:%s\n' % VF(data['lastname']))

        # PHOTO (inline)
        if data['image']:
            encoded_data = base64.encodestring(data['image'])
            out.write('PHOTO;BASE64:\n')
            for line in encoded_data.split('\n'):
                out.write('  ' + line)
            out.write('\n')

        # EMAIL;TYPE=INTERNET
        if data['email']:
            out.write('EMAIL;TYPE=INTERNET:%s\n' % data['email'])

        # URL:http://de.wikipedia.org/
        if data['url']:
            out.write('URL;WORK:%s\n' % data['url'])
        
        # TEL;TYPE=PREF;TYPE=WORK:030-1234567
        if data['phone']:
            out.write('TEL;TYPE=PREF;TYPE=WORK:%s\n' % data['phone'])
            
        # TEL;TYPE=FAX:030-1234567
        if data['fax']:
            out.write('TEL;TYPE=FAX:%s\n' % data['fax'])
        
        # TEL;TYPE=CELL:+49 1234 56789
        if data['mobile']:
            out.write('TEL;TYPE=CELL:%s\n' % data['mobile'])
        
        # TITLE:SACHBEREICH
        # ROLE:FUNKTION)
        if data['position']:
            content = VF(data['position'])
            out.write('ROLE;LANGUAGE=de;CHARSET=%s:%s\n' % (to_encoding, content))

        # ORG;LANGUAGE=de;CHARSET=utf-8:einrichtung
        if data['institution']:
            content = VF(data['institution'])
            out.write('ORG;LANGUAGE=de;CHARSET=%s:%s\n' % (to_encoding, content))
        
        # REV:2007-01-03T11\:08\:23Z
        date_rev = rfc2445dt(DateTime())
        out.write('REV:%s\n' % (vformat(date_rev) ))

        # NOTE aka Description
        if data['notes']:
            content = '%s\n' % VF(data['notes'])
            out.write('NOTE;LANGUAGE=de;CHARSET=%s:%s' % (to_encoding, content))

        # addresses
        for i, row in enumerate(data.get('addresses', ())):
            content= '%s;;%s;%s;%s;%s;%s' % (VF(row['pobox']), 
                                               VF(row['street']),
                                               VF(row['city']),
                                               VF(row['province']),
                                               VF(row['zipcode']),
                                               VF(row['country'])
                                               )
            if i==0:
                out.write('ADR;CHARSET=%s;TYPE=pref,work:%s\n' % (to_encoding,content))
            else:
                out.write('ADR;CHARSET=%s;TYPE=work:%s\n' % (to_encoding,content))

        return out.getvalue()

    def __call__(self, **kwargs):
        """ vCard output  """
        out = StringIO()
        out.write(VC_HEADER)
        out.write(self.getVCard())
        out.write(VC_FOOTER)
        vcard = out.getvalue()
        self.request.RESPONSE.setHeader('Content-Type', 'text/x-vcard')
        self.request.RESPONSE.setHeader('Content-Length', len(vcard))
        self.request.RESPONSE.setHeader('Content-Disposition', 
                                        'attachment; filename="%s.vcf"' % self.context.getId())
        return n2rn(vcard)




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

import re

from zope.interface import implements
from Products.validation.interfaces.IValidator import IValidator
from Products.validation.validators.RegexValidator import RegexValidator

class VSAddressValidator(object):

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
        self.re_poBox = re.compile('^\d+$')
        self.re_zipCode = re.compile('^\d+$')
        self.re_geoCode = re.compile('^\s*\d+\.\d+\s*,\s*\d+\.\d+\s*$')

    def __call__(self, value, *args, **kwargs):
        error_msg = ""
        for val in value[:-1]:
            if val['poBox']:
                if val['street']:
                    error_msg = "Dont use street and postalBox together"

                if not self.re_poBox.match(val['poBox']):
                    error_msg = "Doesnt seem to be a valid postal-box number"

            if val['zipCode'] and (not self.re_zipCode.match(val['zipCode'])):
                error_msg = "Doesnt seem to be a valid zip-code"

            if val['geocode'] and (not self.re_geoCode.match(val['geocode'])):
                error_msg = "Doesnt seem to be a valid geo-code"

            if not val['city']:
                error_msg = "Missing value for city"

        if error_msg:
            return error_msg
        else:
            return True

isVsAddress = VSAddressValidator(
        'isVSAddress', title="Address Validator",
        description="Validator for Addresses"
    )




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

"""
Definition of the Base content type
"""

import vs.org.validators 

from zope.interface import implements

from Products.Archetypes import atapi
from Products.Archetypes.public import DisplayList
from Products.Archetypes.public import *
from Products.ATContentTypes.content import base
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from archetypes.referencebrowserwidget.widget import ReferenceBrowserWidget
from Products.validation import V_REQUIRED

# -*- Message Factory Imported Here -*-
from .. import orgMessageFactory as _

from ..interfaces import IBase
from ..config import PROJECTNAME
from numbers import NumbersMixin

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.atapi import DisplayList

from Products.DataGridField import DataGridField, DataGridWidget, FixedRow
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn

BaseSchema = schemata.ATContentTypeSchema.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

    ReferenceField(
        name='imageReference',
        widget=ReferenceBrowserWidget(
            show_review_state=1,
            allow_sorting=1,
            label=_(u'image_reference',),
        ),
        allowed_types=('Image',),
        multiValued=0,
        relationship='withImage'
    ),

    atapi.StringField(
        'url',
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_url"),
            description=_(u"help_url"),
        ),
        validators=('isURL',),
    ),

    atapi.StringField(
        'email',
        required=True,
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_email"),
            description=_(u"help_email"),
        ),
        validators=('isEmail',),
    ),

    DataGridField(
        'numbers',
        storage=atapi.AnnotationStorage(),
        default=(),
        fixed_rows = [
            FixedRow(keyColumn='type', initialData=dict(type='', externally_visible=False, number='')),
        ],
        columns=('type', 'number', 'externally_visible'),
        widget=DataGridWidget(
            label=_(u"label_numbers"),
            description=_(u"help_numbers"),
            columns = {
                   'type': SelectColumn(_(u'label_number_type'), vocabulary='getNumbersVocabulary'),
                   'number': Column(_(u'label_number')),
                   'externally_visible': CheckboxColumn(_(u'label_externally_visible')),
            }
        ),
    ),

))

# Set storage on fields copied from ATContentTypeSchema, making sure
# they work well with the python bridge properties.

BaseSchema['title'].storage = atapi.AnnotationStorage()
BaseSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(BaseSchema, moveDiscussion=False)




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

"""
Define a browser view for the Department content type. In the FTI
configured in profiles/default/types/Department.xml, this is being set as the default
view of that content type.
"""

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class DepartmentView(BrowserView):
    """ Default view, one department """

    __call__ = ViewPageTemplateFile('department_view.pt')




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: department.py 2983 2011-03-16 11:36:36Z carsten $

"""
Definition of the Department content type
"""

from AccessControl import ClassSecurityInfo

from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from zope.app.container.interfaces import INameChooser
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implements

from archetypes.referencebrowserwidget.widget import ReferenceBrowserWidget
from compatible import atapi
from Products.Archetypes.interfaces import IObjectInitializedEvent
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content import base
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from Products.CMFCore.permissions import View

# -*- Message Factory Imported Here -*-
from .. import orgMessageFactory as _

from ..interfaces import IBase
from ..interfaces import IDepartment
from ..config import PROJECTNAME
from ..config import DEPARTMENTS_PORTLET_COLUMN
from ..portlets import departmentportlet
from base import BaseSchema
from numbers import NumbersMixin

DepartmentSchema = folder.ATFolderSchema.copy() + ConstrainTypesMixinSchema.copy() + BaseSchema.copy() + atapi.Schema((

    atapi.TextField('text',
              required=False,
              searchable=True,
              primary=True,
              storage = atapi.AnnotationStorage(),
              validators = ('isTidyHtmlWithCleanup',),
              default_output_type = 'text/x-html-safe',
              widget = atapi.RichWidget(
                        description='',
                        label=_(u'label_body_text', default=u'Body Text'),
                        rows=25,
                        allow_file_upload=zconf.ATDocument.allow_document_upload),
    ),

    atapi.ReferenceField(
        name='employees',
        multiValued=1,
        allowed_types=('Employee',),
        relationship='isEmployee',
        widget=ReferenceBrowserWidget(
            show_review_state=1,
            allow_sorting=1,
            label=_(u'label_employees',),
        ),
    ),

    atapi.TextField(
        'buildingsection',
        required=True,
        searchable=True,
        storage=atapi.AnnotationStorage(),
        widget=atapi.StringWidget(
            label=_(u"label_building_section"),
            description=_(u"help_building_section"),
        ),
    ),


    atapi.LinesField(
        'specialisation',
        accessor='Specialisation',
        required=False,
        searchable=True,
        storage=atapi.AnnotationStorage(),
        widget=atapi.KeywordWidget(
            label=_(u"label_specialisation"),
            description=_(u"help_specialisation"),
        ),
    ),

))

DepartmentSchema['title'].storage = atapi.AnnotationStorage()
DepartmentSchema['description'].storage = atapi.AnnotationStorage()
DepartmentSchema['text'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    DepartmentSchema,
    folderish=True,
    moveDiscussion=False
)


class Department(folder.ATFolder, NumbersMixin):
    """Department of an institution"""

    implements(IDepartment, IBase)
    meta_type = 'Department'
    numbers_vocabulary = 'number_types_department'
    security = ClassSecurityInfo()
    schema = DepartmentSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    buildingsection = atapi.ATFieldProperty('buildingsection')
    specialisation = atapi.ATFieldProperty('specialisation')

    security.declareProtected(View, 'getDepartment')
    def getDepartment(self):
        """ return the department """
        return self

    def at_post_edit_script(self, *args, **kw):
        """ Filter out empty number fields """
        super(Department, self).at_post_edit_script(*args, **kw)
        numbers = self.getNumbers()
        numbers = [d for d in numbers if d['number']]
        self.setNumbers(numbers)

atapi.registerType(Department, PROJECTNAME)

@adapter(IDepartment, IObjectInitializedEvent)
def setup_department_portlets(obj, event):
    column = getUtility(IPortletManager, name=DEPARTMENTS_PORTLET_COLUMN)
    manager = getMultiAdapter((obj, column,), IPortletAssignmentMapping)
    
    has_department = False
    for value in manager.values():
        if value.__name__ == 'label_departmentportlet':
            has_department = True
    if not has_department:
        assignment_department = departmentportlet.DepartmentPortletAssignment(department_uid=obj.UID())
    
        chooser = INameChooser(manager)
        department_name  = chooser.chooseName(None, assignment_department) 
        manager[chooser.chooseName(None, assignment_department)] = assignment_department




################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

"""
Employee folder search view 
"""


from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

class EmployeefolderView(BrowserView):
    """Default view of an employeefolder """

    __call__ = ViewPageTemplateFile('employeefolder_view.pt')

    def searchEmployees(self):
        """ Search form callback """
        catalog = getToolByName(self.context, 'portal_catalog')
        vs_sheet = getToolByName(getToolByName(self.context, 'portal_properties'), 'vsorg_properties')
        lastname = self.request.get('lastname', '')
        if lastname:
            # if requested: strip of wildcards
            if vs_sheet.employeeFolderExactSearch:
                lastname = lastname.replace('?', '')
                lastname = lastname.replace('.', '')
                lastname = lastname.replace('*', '')
            return catalog(portal_type='Employee',
                           path='/'.join(self.context.getPhysicalPath()),
                           lastname=lastname)   
        else:
            return ()



# -*- coding: utf-8 -*-
"""
This module contains the tool of vs.org
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.0.2'

long_description = (
    read('README.txt')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Detailed Documentation\n'
    '**********************\n'
    + '\n' +
    read('vs', 'org', 'README.txt')
    + '\n' +
    'Contributors\n'
    '************\n'
    + '\n' +
    read('CONTRIBUTORS.txt')
    + '\n' +
    'Download\n'
    '********\n'
    + '\n' +
    read('TODO.txt')
    + '\n' +
    'Todo\n'
    '****\n')

tests_require = ['zope.testing']

setup(name='vs.org',
      version=version,
      description="Representation of organisational structures",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Plone',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='Zope Plone ',
      author='Veit Schiele',
      author_email='vs.org@veit-schiele.de',
      url='http://svn.plone.org/svn/collective/vs.org/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['vs', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'pycountry',
                        'archetypes.referencebrowserwidget',
                        'Products.DataGridField',
                        'Products.MasterSelectWidget',
                        'Products.ATVocabularyManager',
                        ],
      tests_require=tests_require,
      extras_require = {
              'test': [ 'plone.app.testing', 'unittest2' ]
              },
#      test_suite='vs.org.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      setup_requires=["PasteScript"],
      paster_plugins=["ZopeSkel"],
      )



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.publisher.interfaces import IPublishTraverse
from zope.interface import implements
from ZPublisher.BaseRequest import DefaultPublishTraverse

class ImageTraverser(DefaultPublishTraverse):

    implements(IPublishTraverse)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    # check __bobo_traverse_ in ATImage
    def publishTraverse(self, request, name):
        """ traverse to image
            image -> field 'image' (original)
            image_scale -> field 'image' (scale)
            image_fieldid -> field fieldid (original)
            image_fieldid_scale -> field fieldid (scale)
            excluding image_view and image_view fullscreen to make standard-views work
        """
        if name.startswith('image') and not name.startswith('image_view'):
            image = None
            namelist = name.split('_')
            l = len(namelist)
            # image
            if l == 1:
                field = self.context.getField(namelist[0])
                image = field.getScale(self.context)
            # image_scale or image_fieldid
            elif l == 2:
                field = self.context.getField(namelist[1])
                # image_scale etc. (old-style)
                if not field:
                    field = self.context.getField(namelist[0])
                    scalename = namelist[1]
                    if scalename in field.getAvailableSizes(self.context).keys():
                        image = field.getScale(self.context,scale=scalename)
                #image_image
                if not image:
                    image = field.getScale(self.context)
            # image_fieldid_scale
            if l == 3: 
                field = self.context.getField(namelist[1])
                scalename = namelist[2]
                if scalename in field.getAvailableSizes(self.context).keys():
                    image = field.getScale(self.context,scale=scalename)
                else:
                    image = field.getScale(self.context)

            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image
        # default
        return super(ImageTraverser, self).publishTraverse(request, name)



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: institutionportlet.py 2981 2011-03-16 10:54:12Z carsten $

from zope import schema
from zope.formlib import form
from zope.interface import Interface
from zope.interface import implements
from zope.i18nmessageid import MessageFactory

from plone.app.portlets.portlets import base
from plone.memoize import ram
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.cache import render_cachekey

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from .. import orgMessageFactory as _
from ..browser.imagesupport import ImageView
from ..browser.staffsort import StaffSortView
from ..browser.vbbsupport import VBBView
from ..interfaces import IInstitution, IBase


class IInstitutionPortlet(IPortletDataProvider):
    """A portlet

    It inherits from IPortletDataProvider because for this portlet, the
    data that is being rendered and the portlet assignment itself are the
    same.
    """

    institution_uid = schema.Choice(
        vocabulary = u'source_institutions',
        title    = _(u'box_label_institution'),
        required = True,
    )

class InstitutionPortletAssignment(base.Assignment):
    """Portlet assignment.

    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(IInstitutionPortlet,IBase)

    institution_uid = ''

    def __init__(self, institution_uid=''):
        self.institution_uid = institution_uid

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen.
        """
        return _(u'box_label_institutionportlet') 


class Renderer(base.Renderer):
    """Portlet renderer.

    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """

    render = ViewPageTemplateFile('institutionportlet.pt')

    _template = ViewPageTemplateFile('institutionportlet.pt')

    # @ram.cache(render_cachekey)
    def render(self):
        return xhtml_compress(self._template())

    @property
    def available(self):
        # return self._data() is not None
        return 1

    def getInstitution(self):
        return self._data()

    @memoize
    def _data(self):
        institution=None
        instdic=None
        context = aq_inner(self.context)
        ref_catalog = getToolByName(context, 'reference_catalog')
        uid = self.data.institution_uid
        institution = ref_catalog.lookupObject(uid)
        instdic={}
        if institution:
            vbb_view = VBBView(institution, None)
            staff_view = StaffSortView(institution, None)
            image_view = ImageView(institution, None)
            instdic={
                'title': institution.Title(),
                'addresses': institution.getAddresses(),
#                'state': institution.state(),
                'email': institution.getEmail(),
                'email_to_show': institution.getEmail().replace('@', ' @ '),
                'homepage': institution.getUrl(),
                'url': institution.absolute_url(),
                'vbb': vbb_view.getVBBUrl(),
                'staff': staff_view.getSortedStaff(),
                'image': image_view.getReferenceImage(),
                'institution': institution,
            }
        return instdic


class AddForm(base.AddForm):
    """Portlet add form.

    This is registered in configure.zcml. The form_fields variable tells
    zope.formlib which fields to display. The create() method actually
    constructs the assignment that is being added.
    """
    form_fields = form.Fields(IInstitutionPortlet)

    def create(self):
        return InstitutionPortletAssignment(institution_uid=self.context.UID())

class EditForm(base.EditForm):
    """Portlet edit form.

    This is registered with configure.zcml. The form_fields variable tells
    zope.formlib which fields to display.
    """
    form_fields = form.Fields(IInstitutionPortlet)



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id: employeefolder.py 2981 2011-03-16 10:54:12Z carsten $

"""
Definition of the Employeefolder content type
"""

from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize

from AccessControl import ClassSecurityInfo
from compatible import atapi
from Products.ATContentTypes.content import folder
from Products.CMFCore.permissions import View
from Products.ATContentTypes.content import schemata
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from ..interfaces import IBase
from ..interfaces import IEmployeefolder
from ..config import PROJECTNAME

EmployeefolderSchema = folder.ATFolderSchema.copy() + atapi.Schema((

))

EmployeefolderSchema['title'].storage = atapi.AnnotationStorage()
EmployeefolderSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    EmployeefolderSchema,
    folderish=True,
    moveDiscussion=False
)


class Employeefolder(folder.ATFolder):
    """Employees of an institution or department"""
    implements(IEmployeefolder,IBase)

    meta_type = "Employeefolder"
    schema = EmployeefolderSchema
    security = ClassSecurityInfo()

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    security.declareProtected(View, 'getEmployeeFolder')
    def getEmployeeFolder(self):
        """ return the department """
        return self


class Renderer(base.Renderer):

    # render() will be called to render the portlet

    render = ViewPageTemplateFile('employeefolder.pt')

    # The 'available' property is used to determine if the portlet should
    # be shown.

    @property
    def available(self):
        return len(self._data()) > 0

    # To make the view template as simple as possible, we return dicts with
    # only the necessary information.

    def employeefolder(self):
        for brain in self._data():
            employee = brain.getObject()
            yield dict(title=registrant.name,
                       url=brain.getURL())

    @memoize
    def _data(self):
        context = aq_inner(self.context)
        limit = self.data.count

        query = dict(object_provides = IEmployee.__identifier__)

        query['sort_on'] = 'modified'
        query['sort_order'] = 'reverse'
        query['sort_limit'] = limit
        if not self.data.sitewide:
            query['path'] = '/'.join(context.getPhysicalPath())

        # Ensure that we only get active objects, even if the user would
        # normally have the rights to view inactive objects (as an
        # administrator would)
        query['effectiveRange'] = DateTime()

        catalog = getToolByName(context, 'portal_catalog')
        results = catalog(query)
        employeefolder = results[:limit]

        return employeefolder

atapi.registerType(Employeefolder, PROJECTNAME)



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.interface import implements

from Products.Five.browser import BrowserView
from plone.memoize.instance import memoize

from interfaces import IMapView
from ..interfaces import IAddressProvider


class MapView(BrowserView):
    """ Google Maps integration """

    def __init__(self, context, request):
        self.context = context
        self.request = request
                                
    def getAddresses(self):
        """ Provide the addresses of the current object (requires
            to implement IAddressProvider.
        """
        adapter = IAddressProvider(self.context)
        return adapter.getAddresses()



################################################################
# vs.org (C) 2011, Veit Schiele
################################################################

# $Id$

from zope.interface import Interface
# -*- Additional Imports Here -*-
from zope import schema

from vs.org import orgMessageFactory as _



class IBase(Interface):
    """Common elements for other contenttypes"""

    # -*- schema definition goes here -*-
    image = schema.Bytes(
        title=_(u"label_image"),
        required=False,
        description=_(u"help_image"),
    )

    url = schema.TextLine(
        title=_(u"label_url"),
        required=False,
        description=_(u"label_url"),
    )
#
    email = schema.TextLine(
        title=_(u"label_email"),
        required=False,
        description=_(u"help_email"),
    )
#
    telephone = schema.TextLine(
        title=_(u"label_telephone"),
        required=False,
        description=_(u"help_telephone"),
    )
#
